from engine.pyEng    import *
from engine.pyEngObj import *
from explosion       import *
from math            import *
import random

stars      = []
bluePlace  = []
greenPlace = []
redPlace   = []

def createStars():
    starLenth = 2.0, 2.0, 15.0
    starRot   = 45.0, -45.0, 0.0
    
    for i in range(121):
        star = []
        rot  = i * (pi/180)
            
        for i in range(3):
            r = (i*(2*pi/3)) + (i*(pi/180)) + rot
            tri = []
            for j in range(3):
                x = 1.5 * starLenth[j] * math.cos(starRot[j] + r)
                y = 1.5 * starLenth[j] * math.sin(starRot[j] + r)
                tri.append([x, y])
            star.append(tri)
            
        stars.append(star)
        
def createPlaces():
    bdist = 50
    gdist = 50
    rdist = 50
    
    for i in range(23):
        bplace = []
        gplace = []
        rplace = []
        count  = 0
        for j in range(30):
            rx = (i * (math.pi/5)) + (j * (math.pi/7))
            dx = (bdist * math.cos(rx))
            ry = (i * (math.pi/9)) + (j * (math.pi/11))
            dy = (bdist * math.cos(ry))
            bplace.append([dx, dy, int(round(10 * random.random())), count%2])
            count += 1
            
            rx = (i * (math.pi/7)) + (j * (math.pi/5))
            dx = (gdist * math.cos(rx))
            ry = (i * (math.pi/11)) + (j * (math.pi/9))
            dy = (gdist * math.cos(ry))
            gplace.append([dx, dy, int(round(10 * random.random())), count%2])
            count += 1
            
            rx = (i * (math.pi/9)) + (j * (math.pi/11))
            dx = (rdist * math.cos(rx))
            ry = (i * (math.pi/5)) + (j * (math.pi/7))
            dy = (rdist * math.cos(ry))
            rplace.append([dx, dy, int(round(10 * random.random())), count%2])
            count += 1
            
        bluePlace.append(bplace)
        greenPlace.append(gplace)
        redPlace.append(rplace)
        
createStars()
createPlaces()

class Background(PyEngObj):
    
    def __init__(self):
        PyEngObj.__init__(self)
        
        self.category = 'SCENERY'
        self.name     = 'BACKGROUND'
        self.screen   = engine.get("ScreenDevice")
        self.xDensity = 5
        self.yDensity = 5
        self.chance   = 6
        self.switch   = 0
        self.pi23     = (2*pi)/3
        self.pi180    = 180.0/pi
        
    def tick (self, time, dtime):
        
        if(self.switch == 0):
            self.switch = 1
        else:
            self.switch = 0
                
        bw  = int((1*self.screen.screenWidth)/(self.xDensity-1))
        bcw = round((self.screen.camera[0] - (1*self.screen.screenW2)) - (self.screen.camera[0]%bw), -1)
        bh  = int((1*self.screen.screenHeight)/(self.yDensity-1))
        bch = round((self.screen.camera[1] - (1*self.screen.screenH2)) - (self.screen.camera[1]%bh), -1)
        
        gw  = int((2*self.screen.screenWidth)/(self.xDensity-1))
        gcw = round((self.screen.camera[0] - (2*self.screen.screenW2)) - (self.screen.camera[0]%gw), -1)
        gh  = int((2*self.screen.screenHeight)/(self.yDensity-1))
        gch = round((self.screen.camera[1] - (2*self.screen.screenH2)) - (self.screen.camera[1]%gh), -1)
        
        rw  = int((3*self.screen.screenWidth)/(self.xDensity-1))
        rcw = round((self.screen.camera[0] - (3*self.screen.screenW2)) - (self.screen.camera[0]%rw), -1)
        rh  = int((3*self.screen.screenHeight)/(self.yDensity-1))
        rch = round((self.screen.camera[1] - (3*self.screen.screenH2)) - (self.screen.camera[1]%rh), -1)
                
        for i in range(self.xDensity):
            bx  = int((i * bw) + bcw)
            gx  = int((i * gw) + gcw)
            rx  = int((i * rw) + rcw)

            bxi = bx%23
            gxi = gx%23
            rxi = rx%23
                            
            for j in range(self.yDensity):
                by = int((j * bh) + bch)
                gy = int((j * gh) + gch)
                ry = int((j * rh) + rch)
                
                byi = by%23
                if(bluePlace[bxi][byi][2] > self.chance):
                    if(bluePlace[bxi][byi][3] == self.switch):
                        dx    = bluePlace[bxi][byi][0]
                        dy    = bluePlace[bxi][byi][1]
                        scale = .3 + (.6 * random.random())
                        rot   = self.pi23 * random.random()
                        rot   = int(rot * self.pi180)
                        self.drawStar([bx + dx,  by + dy], rot, [50, 50, 200], scale)
                        
                gyi = gy%23
                if(greenPlace[gxi][gyi][2] > self.chance):
                    if(greenPlace[gxi][gyi][3] == self.switch):
                        dx    = greenPlace[gxi][gyi][0]
                        dy    = greenPlace[gxi][gyi][1]
                        dx   -= (0.5*(gx - self.screen.camera[0]))
                        dy   -= (0.5*(gy - self.screen.camera[1]))
                        scale = .2 + (.4 * random.random())
                        rot   = self.pi23 * random.random()
                        rot   = int(rot * self.pi180)
                        self.drawStar([gx + dx,  gy + dy], rot, [50, 200, 50], scale)
                        
                ryi = ry%23
                if(redPlace[rxi][ryi][2] > self.chance):
                    if(redPlace[rxi][ryi][3] == self.switch):
                        dx     = redPlace[rxi][ryi][0]
                        dy     = redPlace[rxi][ryi][1]
                        dx    -= (.75*(rx - self.screen.camera[0]))
                        dy    -= (.75*(ry - self.screen.camera[1]))
                        scale = .1 + (.2 * random.random())
                        rot   = self.pi23 * random.random()
                        rot   = int(rot * self.pi180)
                        self.drawStar([rx + dx, ry + dy], rot, [200, 50, 50], scale)
    
    def drawStar(self, pos, rot, color, scale):
        for i in stars[rot]:
            p = []
            for j in i:
                p.append([(scale*j[0])+pos[0], (scale*j[1])+pos[1]])
                
            self.screen.drawPoly(p, color, 2)
        