from engine.pyEng import *
from device       import *
from math         import *
import pygame

class renderObject:

    def __init__(self):
        self.category  = 0
        self.color     = [255, 255, 255]
        self.image     = 0
        self.pos       = [0, 0]
        self.radius    = 10
        self.pointList = []
        self.text      = ''
        self.size      = 0

class ScreenDevice(Device):
    
    def __init__(self):
        Device.__init__(self)
        
        self.name = "ScreenDevice"
        self.screenWidth   = 0
        self.screenHeight  = 0
        self.screen        = 0
        self.zBuffer       = {}
        self.camera        = [0, 0]
        self.fonts         = {}
        self.renderObjects = []
        self.configure(800, 600, "Generic Text")
        
        self.movePoints = []
        self.moveDepth  = 10
        
    def configure(self, width, height, text):
        self.screenWidth  = width
        self.screenHeight = height
        self.screenW2     = width/2
        self.screenH2     = height/2
        self.screenCW     = self.camera[0] + self.screenW2
        self.screenCH     = self.camera[1] + self.screenH2
        self.screen       = pygame.display.set_mode([self.screenWidth, self.screenHeight])
        pygame.display.set_caption(text)
        
    def tick(self, time, dTime):
        self.screenCW = self.camera[0] - self.screenW2
        self.screenCH = self.camera[1] - self.screenH2
        self.screen.fill([0,0,0])
        
        self.move()

        #self.drawText([200, 200], 'FPS: ' + str(int(engine.clock.get_fps())), 48, [255, 0, 0])

        l = self.zBuffer.items()
        l.sort()
        for key, value in l:
            for o in value:
                if(o.category == 0):
                    pygame.draw.circle(self.screen, o.color, o.pos, o.radius)
                elif(o.category == 1):
                    pygame.draw.polygon(self.screen, o.color, o.pointList)
                elif(o.category == 2):
                    self.screen.blit(o.image, o.pos)
                elif(o.category == 3):
                    if(self.fonts.has_key(o.size)):
                        t = self.fonts[o.size]
                    else:
                        t = pygame.font.Font(None, o.size)
                    self.screen.blit(t.render(o.text, 1, o.color), o.pos)
                self.renderObjects.append(o)
                
            self.zBuffer[key] = []
        pygame.display.flip()
                
    def render(self, obj, zIndex):
        if(self.zBuffer.has_key(zIndex)):
            l = self.zBuffer[zIndex]
        else:
            l = []
            self.zBuffer[zIndex] = l
        l.append(obj)
        
    def getRenderObject(self):
        if(len(self.renderObjects) > 1):
            l = self.renderObjects.pop()
        else:
            l = renderObject()
        return l
            
    def drawCircle(self, pos, radius, color, zIndex = 0):
        obj = self.getRenderObject()
        obj.category = 0
        obj.color    = color
        obj.pos      = [int(pos[0]-self.screenCW), int(pos[1]-self.screenCH)]
        obj.radius   = int(radius)
        self.render(obj, zIndex)
        
    def drawCircle2(self, pos, radius, color, zIndex = 0):
        obj = self.getRenderObject()
        obj.category = 0
        obj.color    = color
        obj.pos      = [int(pos[0]), int(pos[1])]
        obj.radius   = int(radius)
        self.render(obj, zIndex)
    
    def drawRect(self, pos, height, width, rotation, color, zIndex = 0):
        obj = self.getRenderObject()
        obj.category = 1
        obj.color    = color
        h = float(height)/2
        w = float(width)/2
        r = float(-rotation)
        obj.pointList = []
        obj.pointList.append([(( w*cos(r)) - ( h*sin(r))) + pos[0], (( w*sin(r)) + ( h*cos(r))) + pos[1]])
        obj.pointList.append([((-w*cos(r)) - ( h*sin(r))) + pos[0], ((-w*sin(r)) + ( h*cos(r))) + pos[1]])
        obj.pointList.append([((-w*cos(r)) - (-h*sin(r))) + pos[0], ((-w*sin(r)) + (-h*cos(r))) + pos[1]])
        obj.pointList.append([(( w*cos(r)) - (-h*sin(r))) + pos[0], (( w*sin(r)) + (-h*cos(r))) + pos[1]])
        self.render(obj, zIndex)
        
    def drawPoly(self, points, color, zIndex = 0):
        obj = self.getRenderObject()
        obj.category  = 1
        obj.color     = color
        obj.pointList = []
        for p in points:
            obj.pointList.append([p[0]-self.screenCW ,p[1]-self.screenCH])
        self.render(obj, zIndex)
        
    def drawPoly2(self, points, color, zIndex = 0):
        obj = self.getRenderObject()
        obj.category  = 1
        obj.color     = color
        obj.pointList = []
        for p in points:
            obj.pointList.append([p[0] ,p[1]])
        self.render(obj, zIndex)
        
    def drawSprite(self, pos, rotation, image, zIndex = 0):
        obj = self.getRenderObject()
        obj.category = 2
        obj.image    = pygame.transform.rotate(image, 180*(rotation/math.pi))
        h = obj.image.get_height()/2
        w = obj.image.get_width()/2
        obj.pos = [pos[0]-w, pos[1]-h]
        self.render(obj, zIndex)
        
    def drawText(self, pos, text, size, color, zIndex = 100):
        obj = self.getRenderObject()
        obj.category = 3
        obj.size     = size
        obj.color    = color
        obj.text     = text
        obj.pos = [pos[0], pos[1]]
        self.render(obj, zIndex)
    
    def pushMove(self, x, y):
        self.movePoints.append([x,y])
   
    def move(self):
        if(len(self.movePoints) > self.moveDepth):
            pos = self.movePoints.pop(0)
            self.camera[0] = (self.camera[0] + pos[0])/2
            self.camera[1] = (self.camera[1] + pos[1])/2